# -*- coding: utf-8 -*-

"""Parses ASN.1 value notation into a Python object or JSON without requiring its ASN.1 schema."""

__version__ = '0.2.3'

from asn1vnparser._asn1vnparser import parse_asn1_value, parse_asn1_value_assignment, parse_asn1_value_assignments
from asn1vnparser._grammar import AsnValueAssignment

__all__ = ["AsnValueAssignment", "parse_asn1_value",
           "parse_asn1_value_assignment", "parse_asn1_value_assignments"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `asn1vnparser` package."""


import json
import pathlib
import sys
import test.support
import unittest

import asn1vnparser
from asn1vnparser import cli


class TestAsn1vnparser(unittest.TestCase):
    """Tests for `asn1vnparser` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_json(self):
        """Test conversion to JSON representations."""

        cases_dir = pathlib.Path(__file__).parent / 'cases'

        asn_strs = {
            asn_path.stem: asn_path.read_text()
            for asn_path in cases_dir.glob('*.asn')
        }
        json_strs = {
            json_path.stem: json_path.read_text()
            for json_path in cases_dir.glob('*.json')
        }

        assert set(asn_strs.keys()) == set(json_strs.keys())
        assert len(asn_strs) > 0

        for key in asn_strs:
            with self.subTest(key=key):
                res_json = asn1vnparser.parse_asn1_value_assignment(
                    asn_strs[key], as_json=True)
                res_py = json.loads(res_json)
                self.maxDiff = None
                self.assertEqual(res_py, json.loads(json_strs[key]))

    def test_command_line_interface(self):
        """Test the CLI."""

        with self.subTest('parsing value assignment into python obj dump'):
            with test.support.captured_stdout() as stdout, \
                    test.support.captured_stdin() as stdin:
                stdin.write(r'value-1 Type-1 ::= {f1 v1, f2 a2 : 2}')
                stdin.seek(0)
                cli.main([])
                self.assertEqual(
                    stdout.getvalue(),
                    "{'value_name': 'value-1', 'type_name': 'Type-1', " +
                    "'value': {'f1': 'v1', 'f2': {'a2': 2}}}\n"
                )

        with self.subTest('parsing value assignment into JSON'):
            with test.support.captured_stdout() as stdout, \
                    test.support.captured_stdin() as stdin:
                stdin.write(r'value-1 Type-1 ::= {a1 : enum1, a2 : 2}')
                stdin.seek(0)
                cli.main(["-j"])
                self.assertEqual(
                    stdout.getvalue(),
                    '{"value_name": "value-1", "type_name": "Type-1", "value": [{"a1": "enum1"}, {"a2": 2}]}\n'
                )

        with self.subTest('parsing a single value into JSON'):
            with test.support.captured_stdout() as stdout, \
                    test.support.captured_stdin() as stdin:
                stdin.write(r'SomeType : {1, 2, 3}')
                stdin.seek(0)
                cli.main(["-jv"])
                self.assertEqual(
                    stdout.getvalue(),
                    '{"SomeType": [1, 2, 3]}\n')

        with self.subTest('parsing multiple value assignments into JSON'):
            with test.support.captured_stdout() as stdout, \
                    test.support.captured_stdin() as stdin:
                stdin.write('value-1 Type-1 ::= 3\nvalue-2 Type-2 ::= TRUE')
                stdin.seek(0)
                cli.main(["-jm"])
                self.assertEqual(
                    stdout.getvalue(),
                    '[{"value_name": "value-1", "type_name": "Type-1", "value": 3}, {"value_name": "value-2", "type_name": "Type-2", "value": true}]\n'
                )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Integrates doctests into unittest (and `setup.py test`)."""


import doctest
import unittest

from asn1vnparser import _asn1vnparser, _grammar, jsonencoder


def load_tests(_loader, tests, _ignore):
    tests.addTests(doctest.DocTestSuite(_asn1vnparser))
    tests.addTests(doctest.DocTestSuite(_grammar))
    tests.addTests(doctest.DocTestSuite(jsonencoder))
    return tests



# -*- coding: utf-8 -*-

"""Console script for asn1vnparser."""
import argparse
import json
import pathlib
import sys

import asn1vnparser


def main(orig_args=None):
    """Console script for asn1vnparser.

    See `python cli.py --help`."""
    parser = argparse.ArgumentParser(
        prog='asn1vnparser',
        description='Prints out Python object (repr) or JSON for an ASN.1 value.')
    g_multi_not_support_value_only = parser.add_mutually_exclusive_group()
    parser.add_argument('input_file', nargs='?', default='-',
                        help='input file path. Pass - (a single hypehn) to read from stdin.')
    parser.add_argument('-e', '--encoding', default=None,
                        help='input file encoding')
    g_multi_not_support_value_only.add_argument('-m', '--multi', action='store_true',
                                                help='parses multiple ASN.1 value assignments into a list. When -m is specified, -v is not supported.')
    g_multi_not_support_value_only.add_argument('-v', '--value_only', action='store_true',
                                                help='parses an ASN.1 value (e.g. "3"), not a value assignment (e.g. "value Type ::= 3")')
    parser.add_argument('-j', '--json', action='store_true',
                        help='prints out JSON obejct. By default, this program prints out repr() of the resulting python object.')
    parser.add_argument('-o', '--output_file', nargs='?', default=None,
                        help='output file path. By default this program prints the result to stdout.')
    parser.add_argument('-f', '--force', action='store_true',
                        help='force to overwrite an existing file.')

    args = parser.parse_args(orig_args)

    if args.input_file == '-':
        input_str = sys.stdin.read()
    else:
        input_str = pathlib.Path(args.input_file).read_text(
            encoding=args.encoding)

    if args.value_only:
        ret_value = asn1vnparser.parse_asn1_value(input_str, as_json=args.json)
    elif args.multi:
        ret_value = asn1vnparser.parse_asn1_value_assignments(
            input_str, as_json=args.json)
    else:
        ret_value = asn1vnparser.parse_asn1_value_assignment(
            input_str, as_json=args.json)

    if args.json:
        ret_str = ret_value
    else:
        ret_str = repr(ret_value)

    if args.output_file:
        output_path = pathlib.Path(args.output_file)
        if args.force or not output_path.exists():
            output_path.write_text(ret_str)
        else:
            raise FileExistsError(
                'output file already exists: {}'.format(args.output_file))
    else:
        print(ret_str)

    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



# -*- coding: utf-8 -*-

"""Tokenizers and parsers."""

import re
from typing import Any, ClassVar, Dict, List, Type, Union

import bitarray
import pyparsing as pp

# --- X.680 --- #
# 12.2
typereference = pp.Word(initChars=pp.alphas.upper(),
                        bodyChars=pp.alphanums + '-')

# 12.3, 12.4
identifier = pp.Regex(r'[a-z]([a-zA-Z0-9\-]*[a-zA-Z0-9])?')
valuereference = identifier

# 12.8
number = pp.Regex(r'(0|[1-9][0-9]*)')

# 12.9
realnumber = pp.Regex(r'[0-9]+(\.([0-9]+)?)?([eE][\+\-]?(0|[1-9][0-9]+))?')


# --- global --- #

_forwarded_syntaxes = []


# --- classes --- #

class Meta_AddedToForwardedSyntaxes(type):
    """Adds classes into `_forwarded_syntaxes` on definition."""
    def __new__(cls, *args, **kwargs):
        new_class = super().__new__(cls, *args, **kwargs)
        _forwarded_syntaxes.append(new_class)
        return new_class


class AsnDefinition(object, metaclass=Meta_AddedToForwardedSyntaxes):
    """Base class for ASN.1 syntax."""

    # ## Syntax Definitions
    # Use `_raw_syntax` for non-recursive syntax definitions.
    # For recursive definitions,
    #     1. Defer syntax definition with `_raw_syntax = p.Forward()`
    #     2. Define a class method named `get_forwarded_syntax`
    #        with no argument which returns the syntax definition.
    #
    # ## Converting to Python object
    # Conversion can be done in __init__ method.
    # Receive tokens by `def __init__(self, toks: p.ParseResults)`
    # and set `self.value` to the Python object after conversion.

    _raw_syntax = None  # type: ClassVar[Type[pp.ParserElement]]

    @classmethod
    def syntax(cls) -> Type[pp.ParserElement]:
        cls._raw_syntax.setParseAction(cls)
        return cls._raw_syntax

    @classmethod
    def add_forwarded_syntax(cls):
        if hasattr(cls, 'get_forwarded_syntax'):
            cls._raw_syntax << (cls.get_forwarded_syntax())

    def __repr__(self):
        return super().__repr__()[:-1] + ' (value={})>'.format(repr(self.value))


# --- ASN.1 grammar components --- #

class AsnBoolean(AsnDefinition):
    """X.680 18.3"""
    def __init__(self, toks: pp.ParseResults):
        value = {
            'TRUE': True,
            'FALSE': False
        }[toks[0]]
        self.value = value  # type: bool

    _raw_syntax = pp.Keyword('TRUE') | pp.Keyword('FALSE')


class AsnInteger(AsnDefinition):
    """X.680 19.9"""
    def __init__(self, toks: pp.ParseResults):
        if 'SignedNumber' in toks:
            value = int(toks['SignedNumber'])
        elif 'identifier' in toks:
            value = toks['identifier']
        else:
            raise ValueError('unknown tokens for integer:', toks)

        self.value = value  # type: Union[int, str]

    _raw_syntax = (
        # (?!\.) is appended to avoid confision with realnumber
        pp.Regex(r'\-?(0|[1-9][0-9]*)(?!\.)').setResultsName('SignedNumber')
        | identifier.copy().setResultsName('identifier')
    )


class AsnEnumerated(AsnDefinition):
    """X.680 20.8"""
    def __init__(self, toks: pp.ParseResults):
        assert len(toks) == 1
        self.value = str(toks[0])

    _raw_syntax = identifier.copy()


class AsnReal(AsnDefinition):
    """X.680 21.6

    Limitations:
        - SequenceValue not supprted; parsed into dict by AsnSequence
    """
    def __init__(self, toks: pp.ParseResults):
        self.value = float(toks[0])

    _raw_syntax = (
        realnumber.copy()
        | (pp.Suppress(pp.Literal('-')) + realnumber.copy())
        | pp.Keyword('PLUS-INFINITY').setParseAction(lambda: 'inf')
        | pp.Keyword('MINUS-INFINITY').setParseAction(lambda: '-inf')
        | pp.Keyword('NOT-A-NUMBER').setParseAction(lambda: 'nan')
    )


class AsnBitString(AsnDefinition):
    """X.680 22.9

    limitations:
        - hstring not supported; parsed into bytes (not bitarray) 
            by AsnOctetstring
        - { identifierList } not supported; parsed into List[str] 
            by AsnSequenceOf and AsnEnumerated
        - {} not supported; parsed into {} (i.e. `dict()`) by AsnSequence
    """
    def __init__(self, toks: pp.ParseResults):
        if 'bstring' in toks:
            bits = re.sub(r'\s', '', toks['bstring'])
            value = bitarray.bitarray(bits)
        elif 'containing' in toks:
            value = toks['containing'].value
        else:
            raise ValueError('unknown toks for bit string: {}'.format(toks))

        self.value = value  # type: Union[bitarray.bitarray, Any]

    _raw_syntax = pp.Forward()

    @classmethod
    def get_forwarded_syntax(cls):
        return (
            pp.Suppress(pp.Literal('\''))
            + pp.Regex(r'[01\s]*').setResultsName('bstring')
            + pp.Suppress(pp.Literal('\'B'))
        ) | (
            pp.Suppress(pp.Keyword('CONTAINING'))
            + AsnValue.syntax().setResultsName('containing')
        )


class AsnOctetString(AsnDefinition):
    """X.680 23.3

    Limitations:

        - bstring not supported; parsed into bitarray (not bytes) by AsnBitstring
    """
    def __init__(self, s, loc, toks: pp.ParseResults):
        if 'hstring' in toks:
            hexstr = re.sub(r'\s', '', toks['hstring'])
            value = bytes.fromhex(hexstr)
        elif 'containing' in toks:
            value = toks['containing'].value
        else:
            raise ValueError('unknown toks for octet string: {}'.format(toks))

        self.value = value  # type: Union[bytes, Any]

    _raw_syntax = pp.Forward()

    @classmethod
    def get_forwarded_syntax(cls):
        return (
            pp.Suppress(pp.Literal('\''))
            + pp.Regex(r'([0-9A-Fa-f]{2}\s*)*').setResultsName('hstring')
            + pp.Suppress(pp.Literal('\'H'))
        ) | (
            pp.Suppress(pp.Keyword('CONTAINING'))
            + AsnValue.syntax().setResultsName('containing')
        )


class AsnNull(AsnDefinition):
    """X.680 24.3"""
    def __init__(self, toks: pp.ParseResults):
        self.value = None

    _raw_syntax = pp.Keyword('NULL')


class AsnSequence(AsnDefinition):
    """X.680 25.18"""
    def __init__(self, toks: pp.ParseResults):
        self.value = dict(
            (tok[0], tok[1].value) for tok in toks
        )  # type: Dict[str, Any]

    _raw_syntax = pp.Forward()

    @classmethod
    def get_forwarded_syntax(cls):
        lbrace = pp.Suppress(pp.Literal('{'))
        rbrace = pp.Suppress(pp.Literal('}'))
        comma = pp.Suppress(pp.Literal(','))
        empty = lbrace.copy() + rbrace.copy()

        content = pp.Group(identifier.copy() + AsnValue.syntax())
        contents = content + pp.ZeroOrMore(comma + content)
        non_empty = lbrace.copy() + contents + rbrace.copy()
        return empty | non_empty


class AsnSequenceOf(AsnDefinition):
    """X.680 26.3

    Limitations:

        - NamedValueList not supported; parsed into dict by AsnSequence
    """
    def __init__(self, toks: pp.ParseResults):
        self.value = [tok.value for tok in toks]  # type: List[Any]

    _raw_syntax = pp.Forward()

    @classmethod
    def get_forwarded_syntax(cls):
        lbrace = pp.Suppress(pp.Literal('{'))
        rbrace = pp.Suppress(pp.Literal('}'))
        comma = pp.Suppress(pp.Literal(','))
        empty = lbrace.copy() + rbrace.copy()

        content = AsnValue.syntax()
        contents = content + pp.ZeroOrMore(comma + content)
        non_empty = lbrace.copy() + contents + rbrace.copy()
        return empty | non_empty


class AsnChoice(AsnDefinition):
    """X.680 29.11"""
    def __init__(self, toks: pp.ParseResults):
        assert len(toks) == 2
        self.value = {toks[0]: toks[1].value}  # type: Dict[str, Any]

    _raw_syntax = pp.Forward()

    @classmethod
    def get_forwarded_syntax(cls):
        return (
            identifier.copy()
            + pp.Suppress(pp.Literal(':'))
            + AsnValue.syntax()
        )


class AsnObjectIdentifier(AsnDefinition):
    """X.680 32.3

    Limitations:

        - Only NumberForm supported
    """
    def __init__(self, toks: pp.ParseResults):
        self.value = [int(tok) for tok in toks]  # type: List[int]

    _raw_syntax = (
        pp.Suppress(pp.Literal(r'{}'))
        | (
            pp.Suppress(pp.Literal('{'))
            + pp.OneOrMore(number.copy())
            + pp.Suppress(pp.Literal('}'))
        )
    )


class AsnCString(AsnDefinition):
    """X.680 41.8 and 12.14

    Limitations:

        - character string forms other than `cstring` are not supported
        - following specification in 12.14 is not respected
            - The "cstring" may span more than one line of text, 
                in which case the character string being represented 
                shall not include spacing characters in the position 
                prior to or following the end of line in the "cstring".

    """
    def __init__(self, toks: pp.ParseResults):
        assert len(toks) == 1
        tok = toks[0]
        self.value = tok.replace("\"\"", "\"")  # type: str

    _raw_syntax = (
        pp.Suppress(pp.Literal("\""))
        + pp.Regex(r'([^\"]|\"\")*')
        + pp.Suppress(pp.Literal("\""))
    )


class AsnOpenTypeFieldVal(AsnDefinition):
    """X.681 14.6"""
    def __init__(self, toks: pp.ParseResults):
        assert len(toks) == 2
        self.value = {toks[0]: toks[1].value}  # type: Dict[str, Any]

    _raw_syntax = pp.Forward()

    @classmethod
    def get_forwarded_syntax(cls):
        return (
            typereference.copy()
            + pp.Suppress(pp.Literal(':'))
            + AsnValue.syntax()
        )


class AsnValue(AsnDefinition):
    """X.680 17.7

    Limitations:

        - Only supports value forms in get_forwarded_syntax
    """
    def __init__(self, s, loc, toks: pp.ParseResults):
        assert isinstance(toks, pp.ParseResults)
        self.value = toks[0].value  # type: Any

    _raw_syntax = pp.Forward()

    @classmethod
    def get_forwarded_syntax(cls):
        # note that this is composed by `|` operator,
        # i.e. `pyparsing.MatchFirst`
        return (
            # compound types
            AsnChoice.syntax()
            | AsnOpenTypeFieldVal.syntax()
            | AsnSequence.syntax()
            | AsnSequenceOf.syntax()
            | AsnObjectIdentifier.syntax()
            # not compound types
            | AsnNull.syntax()
            | AsnBoolean.syntax()
            | AsnInteger.syntax()
            | AsnReal.syntax()
            | AsnBitString.syntax()
            | AsnOctetString.syntax()
            | AsnCString.syntax()
            | AsnEnumerated.syntax()
        )


class AsnValueAssignment(AsnDefinition):
    """Object that represents ASN.1 value assignment (X.680 16.2).
    
    Parses::

        value_name TypeName ::= value

    Limitations:

        - Considers anything that matches `typereference` as `Type`
    
    Attributes:
        value_name (str): name of the assigned value.
        type_name (str): name of the value's type.
        value (Any): Python object that holds the value.
    """

    def __init__(self, toks: pp.ParseResults):
        assert len(toks) == 3
        self.value_name = str(toks[0])
        self.type_name = str(toks[1])
        self.value = toks[2].value  # type: Any

    def __str__(self):
        return str(self.__dict__)

    __repr__ = __str__

    _raw_syntax = (
        valuereference.copy() + typereference.copy() +
        pp.Suppress('::=') + AsnValue.syntax()
    )


for _i in _forwarded_syntaxes:
    _i.add_forwarded_syntax()

# to be exported
value_syntax = pp.StringStart() + AsnValue.syntax()
value_assignment_syntax = pp.StringStart() + AsnValueAssignment.syntax()



# -*- coding: utf-8 -*-

"""Main module."""

from typing import Any, List, Tuple, Union

import pyparsing as pp

from asn1vnparser import _grammar, jsonencoder


def parse_asn1_value(
        asn1_value: str,
        as_json: bool = False,
        parse_all: bool = True
) -> Union[Any, str, Tuple[Any, str], Tuple[str, str]]:
    r"""Converts ASN.1 value (mainly BuiltinValue in ITU-T Rec. X.680) to Python object.

    Args:
        asn1_value: a ``str`` representing ASN.1 value notation(s).
        as_json: Returns JSON when ``True``.
        parse_all: When True, it is assumed that the whole ``asn1_value``
            represents an ASN.1 value.

    Returns:

        - When ``parse_all`` is True, ``result``.
        - When ``parse_all`` is False, a tuple of ``(result, rest)``,

        where ``result`` is:

        - When `as_json` = False, a Python object representation of ``asn1_value``
        - When `as_json` = True, a JSON representation of ``asn1_value`` as `str`

        and ``rest`` is the remaining `str` from ``asn1_value`` 
        after reading the value notation.

    Examples:

        >>> parse_asn1_value("123")  # INTEGER
        123

        >>> parse_asn1_value("{TRUE, FALSE}")  # SEQUENCE OF
        [True, False]

        >>> parse_asn1_value(
        ...     "{field1 NULL, field2 '1204'H, field3 '01011010 01'B}"
        ...     )  # SEQUENCE
        {'field1': None, 'field2': b'\x12\x04', 'field3': bitarray('0101101001')}

        >>> parse_asn1_value("alt1 : enum1")  # CHOICE
        {'alt1': 'enum1'}

        >>> parse_asn1_value("alt1: enum1", as_json=True)  # as_json
        '{"alt1": "enum1"}'

        >>> parse_asn1_value("blah blah")  # raises when not foune  # doctest: +ELLIPSIS
        Traceback (most recent call last):
        ...
        pyparsing.ParseException: ...

        >>> parse_asn1_value("alt1: enum1\n---this is remaining string---", as_json=True)  # by default: parse_all=True. # doctest: +ELLIPSIS
        Traceback (most recent call last):
        ...
        pyparsing.ParseException: Expected end of text...

        >>> parse_asn1_value("alt1: enum1\n---this is remaining string---", as_json=True, parse_all=False)  # as_json
        ('{"alt1": "enum1"}', '\n---this is remaining string---')
    """

    if parse_all:
        result = _grammar.value_syntax.parseString(asn1_value, parseAll=True)
        obj = result[0].value
        if as_json:
            return jsonencoder.JSONEncoder().encode(obj)
        else:
            return obj
    else:
        gen = _grammar.value_syntax.scanString(asn1_value, maxMatches=1)
        results = list(gen)
        if len(results) == 0:
            raise pp.ParseException('value notation not found')

        result, start, end = results[0]
        obj = result[0].value
        rest = asn1_value[end:]

        if as_json:
            return (jsonencoder.JSONEncoder().encode(obj), rest)
        else:
            return (obj, rest)


def parse_asn1_value_assignment(
        asn1_val_assignment: str,
        as_json: bool = False,
        parse_all: bool = True
) -> Union[
    _grammar.AsnValueAssignment,
    str,
    Tuple[_grammar.AsnValueAssignment, str],
    Tuple[str, str]
]:
    r"""Converts ASN.1 value assignment to Python object.

    Args:
        asn1_val_assignment: A ``str`` representing ASN.1 value assignment(s).
        as_json: Returns JSON when ``True``.
        parse_all: When True, it is assumed that the whole ``asn1_val_assignment``
            represents an ASN.1 value assignment.

    Returns:

        - When ``parse_all`` is True, ``result``.
        - When ``parse_all`` is False, a tuple of ``(result, rest)``,

        where ``result`` is:

        - When `as_json` = False, parsing result as ``AsnValueAssignment``.
        - When `as_json` = True, a JSON representation of ``asn1_val_assignment`` as ``str``

        and ``rest`` is the remaining `str` from ``asn1_val_assinment`` 
        after reading the value assignment.

    Examples:

        >>> parse_asn1_value_assignment("valuename Typename ::= 1234")
        {'value_name': 'valuename', 'type_name': 'Typename', 'value': 1234}

        >>> parse_asn1_value_assignment("nonsense")  # raises when not found  # doctest: +ELLIPSIS
        Traceback (most recent call last):
        ...
        pyparsing.ParseException: ...

        >>> parse_asn1_value_assignment("valuename Typename ::= 1234\n...remainremain...")  # doctest: +ELLIPSIS
        Traceback (most recent call last):
        ...
        pyparsing.ParseException: Expected end of text...

        >>> parse_asn1_value_assignment("valuename Typename ::= 1234\n...remainremain...", parse_all=False)
        ({'value_name': 'valuename', 'type_name': 'Typename', 'value': 1234}, '\n...remainremain...')
    """

    if parse_all:
        result = _grammar.value_assignment_syntax.parseString(
            asn1_val_assignment, parseAll=parse_all)
        obj = result[0]

        if as_json:
            return jsonencoder.JSONEncoder().encode(obj.__dict__)
        else:
            return obj

    else:
        gen = _grammar.value_assignment_syntax.scanString(
            asn1_val_assignment, maxMatches=1)
        results = list(gen)
        if len(results) == 0:
            raise pp.ParseException('value assignment not found')

        result, start, end = results[0]
        obj = result[0]
        rest = asn1_val_assignment[end:]
        if as_json:
            return (jsonencoder.JSONEncoder().encode(obj.__dict__), rest)
        else:
            return (obj, rest)


def parse_asn1_value_assignments(
        asn1_val_assignments: str,
        as_json: bool = False) -> List[Union[_grammar.AsnValueAssignment, str]]:
    r"""Parses multiple ASN.1 value assignments.

    Examples:

        >>> parse_asn1_value_assignments('value1 INTEGER ::= 1\nvalue2 Type2 ::= the-value')
        [{'value_name': 'value1', 'type_name': 'INTEGER', 'value': 1}, {'value_name': 'value2', 'type_name': 'Type2', 'value': 'the-value'}]

        >>> parse_asn1_value_assignments('nonsense')  # returns [] when not found
        []

        >>> parse_asn1_value_assignments('value1 INTEGER ::= 1\nvalue2 Type2 ::= the-value', as_json=True)
        '[{"value_name": "value1", "type_name": "INTEGER", "value": 1}, {"value_name": "value2", "type_name": "Type2", "value": "the-value"}]'

        >>> parse_asn1_value_assignments('nonsense', as_json=True)  # returns '[]' when not found
        '[]'
    """

    def gen():
        rest = asn1_val_assignments
        while True:
            try:
                obj, rest = parse_asn1_value_assignment(rest, parse_all=False)
                yield obj
            except pp.ParseException:
                return

    assignments = list(gen())

    if as_json:
        return jsonencoder.JSONEncoder().encode([va.__dict__ for va in assignments])
    else:
        return assignments



# -*- coding: utf-8 -*-

"""Defines `json.JSONEncoder` subclass
that makes parsed object (including bytes and bitarray) JSON-serializable
"""

import bitarray
import json
import sys


class JSONEncoder(json.JSONEncoder):
    """JSON encoder with additional support for bytes and bitarray

    Examples:

        >>> JSONEncoder().encode({"field1": 123})
        '{"field1": 123}'

        >>> JSONEncoder().encode({"field1": b'\x12\x34'})
        '{"field1": "1234"}'

        >>> JSONEncoder().encode({"field1": bitarray.bitarray('01010')})
        '{"field1": "01010"}'

        >>> JSONEncoder(compact_bitarray=True).encode({"field1": bitarray.bitarray('01010')})
        '{"field1": {"value": "50", "length": 5}}'

        >>> JSONEncoder().encode({"field1": {"Type": 567}})
        '{"field1": {"Type": 567}}'
    """

    def __init__(self, compact_bitarray=False, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._compact_bitarray = bool(compact_bitarray)

    def default(self, o):
        if isinstance(o, (bytes, bytearray)):
            return o.hex()
        elif isinstance(o, bitarray.bitarray):
            if self._compact_bitarray:
                return {'value': o.tobytes().hex(), 'length': len(o)}
            else:
                return o.to01()
        else:
            super().default(o)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['bitarray>=0.8.3', 'pyparsing<3,>=2.4.2']

setup_requirements = []

test_requirements = []

setup(
    author="mtannaan",
    author_email='12782884+mtannaan@users.noreply.github.com',
    python_requires='!=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Parses ASN.1 value notation into a Python object or JSON without requiring its ASN.1 schema.",
    entry_points={
        'console_scripts': [
            'asn1vnparser=asn1vnparser.cli:main',
        ],
    },
    install_requires=requirements,
    license="ISC license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='asn1vnparser',
    name='asn1vnparser',
    packages=find_packages(include=['asn1vnparser', 'asn1vnparser.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/mtannaan/asn1vnparser',
    version='0.2.3',
    zip_safe=False,
)
